//=============================================================================
// CNaliFruit.
//=============================================================================
class CNaliFruit expands CHealth
config (MH_A_V2);

var() config bool bGrowWhenSeen;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	bStasis = bGrowWhenSeen;
	HealingAmount=0;
}

event float BotDesireability(Pawn Bot)
{
	if ( HealingAmount < 3 )
		return 0;

	return Super.BotDesireability(Bot);
}

auto state Pickup
{
	function Timer()
	{
		HealingAmount += 2;
		SetTimer(0.5,True);
	}

	function Touch( actor Other )
	{
		local int HealMax;

		if ( !ValidTouch(Other) || (HealingAmount<2) )
			Return;

		if (Level.Game.LocalLog != None)
			Level.Game.LocalLog.LogPickup(Self, Pawn(Other));
		if (Level.Game.WorldLog != None)
			Level.Game.WorldLog.LogPickup(Self, Pawn(Other));

		HealMax = MaxHealth;
		if (Pawn(Other).Health < HealMax) 
		{
			Pawn(Other).Health += HealingAmount;
			if (Pawn(Other).Health > HealMax) Pawn(Other).Health = HealMax;
			Pawn(Other).ClientMessage(PickupMessage$HealingAmount, 'Pickup');
			PlaySound (PickupSound);
			SetRespawn();
			if ( Level.Game.Difficulty > 1 )
				Other.MakeNoise(0.1 * Level.Game.Difficulty);
		}
	}

	simulated function PlayRoot()
	{
		PlayAnim('Root');
	}

	simulated function Grow()
	{
		PlayAnim('Grow',0.036);
	}

	simulated function Waver()
	{
		LoopAnim('Waver',0.1);
	}
Begin:
	AmbientGlow=64;
	bAmbientGlow=True;
	bHidden=False;
	PlayRoot();
	FinishAnim();
	HealingAmount = 0;
	Sleep(FRand()*3);
	SetTimer(8.5,False);
	Grow();
	FinishAnim();
	SetTimer(0.0,False);
	Waver();
}

State Sleeping
{
	ignores Touch;

	simulated function BeginState()
	{
		BecomePickup();
	}
	function EndState()
	{
		local int i;

		bSleepTouch = false;
		for ( i=0; i<4; i++ )
			if ( (Touching[i] != None) && Touching[i].IsA('Pawn') )
				bSleepTouch = true;
	}
Begin:
	bHidden = true;
	Sleep( ReSpawnTime );
	PlaySound( RespawnSound );
	Sleep( Level.Game.PlaySpawnEffect(self) );
	PlayAnim('Root');
	GoToState( 'Pickup' );
}

defaultproperties
{
   MaxHealth=500
   HealingAmount=0
   PickupMessage="You picked up a Nali Healing Fruit +"
   RespawnTime=5.000000
   PickupViewMesh=LodMesh'UnrealShare.NaliFruitMesh'
   RemoteRole=ROLE_SimulatedProxy
   NetUpdateFrequency=5.000000
   Mesh=LodMesh'UnrealShare.NaliFruitMesh'
   AmbientGlow=64
   CollisionHeight=21.000000
   AnimSequence='Root'
}
