class CoopTranslocator expands Translocator;

var bool bCanTranslocate;
var Pawn TargetPawn;
var PlayerReplicationInfo TargetPRI;
var Vector TargetLocation;
var int TargetTime;
var Color TargetColor;
var localized String TranslocateTargetMessage;

replication
{
	reliable if (Role == ROLE_Authority)
		TargetPRI;
	unreliable if (Role == ROLE_Authority)
		TargetLocation;
}

state Active
{
    function BeginState()
    {
        super.BeginState();
        if (bCanTranslocate && TargetPawn == None)
            SelectTarget();
    }
}

event Tick (float delta)
{
//	log ("Coop Translocator Active");
	if ( bDeleteMe ) GoTo JL53;
	if ( Pawn(Owner) == None || Pawn(Owner).bDeleteMe )
	{
		Destroy();
		GoTo JL53;
	}
	TargetTime++;
	if ( Pawn(Owner) != None && Pawn(Owner).Health > 0 && Pawn(Owner).bIsPlayer )
	{
		if ( Pawn(Owner).Weapon != None && Pawn(Owner).Weapon == Self && TargetPawn != None && TargetPawn.Health > 0 && TargetPawn.bIsPlayer )
			TargetLocation = TargetPawn.Location;
	}
	if ( TargetPawn == None || TargetPawn == Pawn(Owner) )
		GoTo JL53;
	if (TargetTime >= 300)
	{
		TargetTime=0;
		TargetPawn=None;
	}
	if ((Level.Game).bGameEnded)
		Destroy();
JL53:
}

function Fire(float Value)
{
	if (bCanTranslocate)
	{
		if (bBotMoveFire)
			GoTo JL68;
		PlayAnim('Throw');
		SelectTarget();
		if ( TargetPawn != None && TargetPawn != Owner )
			Enable('Tick');
		TargetTime = 0;
	}
JL68:
}

function AltFire(float Value)
{
	if (bCanTranslocate)
	{
		if (bBotMoveFire)
			GoTo JL84;
		PlayAnim('Throw', FireAdjust, 0.1);
		GotoState('AltFiring');
		Translocate();
		TargetPawn = None;
		TargetTime = 0;
	}
JL84:
}

function SelectTarget()
{
	local Pawn P;

	if (TargetPawn != None)
	{
		for (P = TargetPawn.NextPawn; P != None; P = P.NextPawn)
		{
			if ( P.PlayerReplicationInfo != None && !P.IsA('Spectator') && P != Owner && !P.IsA('ScriptedPawn') && !P.IsA('TeamCannon') )
			break;
		}
	}
	if (P == None)
	{
		for (P = Level.PawnList; P != None; P = P.NextPawn)
		{
			if ( P.PlayerReplicationInfo != None && !P.IsA('Spectator') && P != Owner && !P.IsA('ScriptedPawn') && !P.IsA('TeamCannon') )
			break;
		}
	}

	if (P != None)
	{
		TargetPawn = P;
		TargetPRI = P.PlayerReplicationInfo;
		TargetLocation = P.Location;
		Owner.PlaySound(FireSound, SLOT_Misc, 4 * Pawn(Owner).SoundDampening);
		Pawn(Owner).ClientMessage(
			TranslocateTargetMessage @
				TargetPawn.PlayerReplicationInfo.PlayerName);
//		Enable('Tick');
	}
	else
	{
		TargetPawn = None;
		TargetPRI = None;
		Owner.PlaySound(AltFireSound, SLOT_Misc, 4 * Pawn(Owner).SoundDampening);
//		Disable('Tick');
		GoTo JL126;
	}
JL126:
}

function Translocate()
{
	local Vector X, Y, Z;
	local Vector Start, Dest;

	bBotMoveFire = false;

	if (TargetPawn == None || TargetPawn.Health < 0)
	{
		Owner.PlaySound(AltFireSound, SLOT_Misc, 4 * Pawn(Owner).SoundDampening);
		GoTo JL165;
	}

	GetAxes(TargetPawn.Rotation, X, Y, Z);
	Dest = TargetPawn.Location
	+ TargetPawn.CollisionRadius * X * -0.9
	+ TargetPawn.CollisionRadius * Y * -0.1
	+ TargetPawn.CollisionHeight * Z * 3;

	Start = Owner.Location;
	if (Owner.SetLocation(Dest))
	{
		if (!Owner.Region.Zone.bWaterZone)
		Owner.SetPhysics(PHYS_Falling);
		Owner.SetRotation(TargetPawn.Rotation);
		Owner.Velocity.X = 0;
		Owner.Velocity.Y = 0;
		Level.Game.PlayTeleportEffect(Owner, true, true);
		SpawnEffect(Start, Dest);
	}
	else
	{
		Owner.PlaySound(AltFireSound, SLOT_Misc, 4 * Pawn(Owner).SoundDampening);
		Disable('Tick');
	}
	bPointing = true;
JL165:
}

simulated function PlayPostSelect()
{
	Super(TournamentWeapon).PlayPostSelect();
}

simulated function PostRender(Canvas C)
{
	local ChallengeHUD HUD;
	local Vector X, Y, Z;
	local Pawn P;
	local Vector Dir;
	local float Dist;
	local float CX, CY;
	local String PlayerName;

	if ( TargetPRI == None )
		GoTo JL204;
	if ( PlayerPawn(Owner) == None )
		GoTo JL204;
	HUD = ChallengeHUD(PlayerPawn(Owner).myHUD);
	if ( HUD == None )
		GoTo JL204;
	C.Font = HUD.MyFonts.GetSmallFont(C.ClipX);
	GetAxes(Owner.Rotation, X, Y, Z);
	Dir = TargetLocation - Location;
	Dist = VSize(Dir);
	Dir /= Dist;
	if (Dir Dot X > 0.7)
	{
		CX = 0.5 * C.ClipX * (1 + 1.4 * (Dir Dot Y));
		CY = 0.5 * C.ClipY * (1 - 1.4 * (Dir Dot Z));
		C.SetPos(CX, CY);
		C.DrawColor = TargetColor;
		C.DrawTextClipped(TargetPRI.PlayerName @ ":" @ int(Dist));
	}
JL204:
}

defaultproperties
{
   bCanTranslocate=True
   TargetColor=(G=255)
   TranslocateTargetMessage="Translocate target is"
   bTTargetOut=True
   bOwnsCrosshair=True
   FireSound=Sound'UnrealShare.Eightball.SeekLock'
   AltFireSound=Sound'UnrealShare.Eightball.SeekLost'
}
