//=============================================================================
// Unreal Tournament Enhanced Statistics - Client ReplicationInfo Class.
//=============================================================================
class UTES_CInfo expands ReplicationInfo;

var int Headshots, Telefrags, Suicides;
var int PlayerID, NumRetries; 		// used to identify the owner
var string Country, NewVar, Pkg;
var Texture CachedFlag;
var Actor IpToCountry;

var bool bReadyToPlay, bAcquired;
var bool bTournament;

var float JoinTime;
var float timeDelta; //used to restore the Time on the scoreboard of reconnectors

var UTES_CInfo HashNext;
var UTES_DMSB HashedAt;

//Server only
var UTES_CInfo LinkNext;
var Pawn PlayerOwner;

replication
{
	reliable if (Role == ROLE_Authority)
		PlayerID, JoinTime, SetTimeDelta, Suicides, Headshots, Telefrags, Country, NewVar, bReadyToPlay;
}

event Spawned()
{
	PlayerOwner = Pawn(Owner);
	Pkg = string(class);
	Pkg = Left( Pkg, InStr(Pkg,"."));
	if(ROLE == ROLE_Authority)
	{
		SetTimer(2.0, True);
		bTournament = DeathMatchPlus(Level.Game).bTournament;
	}
}


simulated event SetInitialState()
{
	bScriptInitialized = true;
	if ( Level.NetMode == NM_Client )
		GotoState( 'ClientOp' );
	else if( InitialState!='' )
		GotoState( InitialState );
	else
		GotoState( 'Auto' );
}

//Loop
simulated state ClientOp
{
Begin:
HashLoop:
	Sleep(0.2);
	if ( HashedAt == none || HashedAt.bDeleteMe )
	{
		HashedAt = none;
		if ( !Hash() )
			Goto('HashLoop');
	}
}

simulated function bool Hash()
{
	local PlayerPawn P;

	ForEach AllActors (class'PlayerPawn', P) //Hash on local games and listen servers
	{
		if ( ViewPort(P.Player) != none )
		{
			if ( UTES_DMSB(P.Scoring) != none )
			{
				HashInfo( UTES_DMSB(P.Scoring) );
				return true;
			}
			break;
		}
	}
	//Automatic return false
}

simulated function HashInfo( UTES_DMSB Score)
{
	local UTES_CInfo Other;
	assert( Score != none ); //Remove this in release version, this crashes the game if this call shouldn't happen
	assert( HashedAt == none); //Hash once prerelease check

	//Remove in release version, crash the game if already hashed
	For ( Other=Score.Hashed[ PlayerID % 32] ; Other != none ; Other=Other.HashNext )
		assert( Other != self );
	HashNext = Score.Hashed[ PlayerID % 32];
	Score.Hashed[ PlayerID % 32] = self;
	HashedAt = Score;
}

simulated function UnHash()
{
	local UTES_CInfo TI;
	assert( HashedAt != none );
	if ( HashedAt.Hashed[ PlayerID % 32] == self )
		HashedAt.Hashed[PlayerID % 32] = HashNext;
	else if ( HashedAt.Hashed[PlayerID % 32] != none )
	{
		For ( TI=HashedAt.Hashed[PlayerID % 32] ; TI.HashNext != none ; TI=TI.HashNext )
			if ( TI.HashNext == self )
			{
				TI.HashNext = HashNext;
				break;
			}
	}
	HashNext = none;
}

//Graceful destroy
simulated event Destroyed()
{
	LinkNext = none;
	PlayerOwner = none;
	if ( HashedAt != none )
		UnHash();
	HashedAt = none;
	IpToCountry = none;
	CachedFlag = none;
}

event Timer()
{
	local string temp;
	local PlayerPawn P;

	if ( bDeleteMe ) return;

	if( Owner == None )
	{
		Destroy();
		return;
	}

	if ( HashedAt == none )
		Hash();

	if ( NumRetries > 10 )
	{
		IpToCountry = None;
		NewVar="Unknown";
		Country="NoFlag";
	}
	if(IpToCountry != None)
	{
		if( Country == "" )
		{
			P=PlayerPawn(Owner);
			if (P != None)
			{
				if ( NetConnection(P.Player) != None)
				{
					temp=P.GetPlayerNetworkAddress();
					temp=Left(temp, InStr(temp, ":"));
					temp=IpToCountry.GetItemName(temp);
					Log (Self.Name@"owned by"@P.GetHumanName()@"getting Country > Answer ::"@temp);
					if(temp == "!Disabled") /* after this return, iptocountry won't resolve anything anyway */
					{
						IpToCountry=None;
						NewVar="Unknown";
						Country="NoFlag";
					}
					else if(Left(temp, 1) != "!") /* good response */
					{
						Country=SelElem(temp, 5);
						NewVar = SelElem(temp, 3);
						bAcquired = True;
						GoTo StopHere;
					}
					else if ( Left(temp, 1) == "!" )
					{
						NewVar="Getting Info...";
						Country="";
						Goto Retry;
					}
					if( Country == "" ) /* the country is probably unknown(maybe LAN), so as the prefix */
					{
//						IpToCountry=None;
						NewVar="Unidentified";
						GoTo Retry;
					}
				}
				else
					IpToCountry=None;
			}
		}
		else
		{
			if ( temp == "!Disabled" )
				IpToCountry=None;
		}
	}
Retry:
	if ( P != None )
		if(bTournament && PlayerPawn(Owner).bReadyToPlay != bReadyToPlay)
		{
			bReadyToPlay = PlayerPawn(Owner).bReadyToPlay; // replicate this variable normally being serverside only
		}
	NumRetries++;
StopHere:
	if ( P != None )
		if( IpToCountry == None || bAcquired )
		{
			SetTimer(0.0, False);
			if ( IpToCountry == None )
			{
				NewVar="Unidentified";
				Country="NoFLag";
			}
		}
End:
}

static final function string SelElem(string Str, int Elem, optional string Char)
{
	local int pos;
	if(Char=="")
		Char=":";

	while(Elem>1)
	{
		Str=Mid(Str, InStr(Str, Char)+1);
		Elem--;
	}
	pos=InStr(Str, Char);
	if(pos != -1)
		Str=Left(Str, pos);
	return Str;
}

simulated function SetTimeDelta(float delta)
{
	//if(Role == ROLE_Authority)
	timeDelta = delta;
}

defaultproperties
{
   PlayerID=-1
   RemoteRole=ROLE_SimulatedProxy
   NetPriority=1.9000000
   Tag=cFL
}
