//================================================================================
// - UnrealTournament Enhanced Statictics - DeathMatch ScoreBoard.
//================================================================================
class UTES_DMSB expands TournamentScoreBoard;

#exec TEXTURE IMPORT NAME=Icon_cpu FILE=Textures\Icon_cpu.bmp Group=HUD MIPS=OFF FLAGS=2
#exec TEXTURE IMPORT NAME=Icon_Con1 FILE=Textures\Icon_Con1.bmp Group=HUD MIPS=OFF FLAGS=2
#exec TEXTURE IMPORT NAME=Icon_Con2 FILE=Textures\Icon_Con2.bmp Group=HUD MIPS=OFF FLAGS=2
#exec TEXTURE IMPORT NAME=Icon_Con3 FILE=Textures\Icon_Con3.bmp Group=HUD MIPS=OFF FLAGS=2
#exec TEXTURE IMPORT NAME=shade2 FILE=Textures\shade2.bmp Group=HUD MIPS=OFF LODSET=2
#exec Texture Import File=Textures\faceless.bmp Name=faceless Group=ScoreBoard MIPS=ON

var color GreenColor, WhiteColor, GoldColor, BlueColor, LightCyanColor, SilverColor, BronzeColor, CyanColor, RedColor,
          LightGreenColor, DarkGreenColor, BlackColor, BColor, ParchmentColor, FrigginDarkColor, PurpleColor;

struct PlayersData
{
   var PlayerReplicationInfo PRI;
   var UTES_CInfo RI;
};
var PlayersData PlayerInfo[32];

var UTES_CInfo Hashed[32];

var int Index;
var UTES_CInfo OwnerRI;
var PlayerPawn PlayerOwner;
var string Spectators[32];

//Find all player info actors spawned prior to this
event PostBeginPlay()
{
	local UTES_CInfo CA;
	Super.PostBeginPlay();
	ForEach AllActors (class'UTES_CInfo', CA)
	{
		if ( CA.PlayerID >= 0 )
			CA.HashInfo( self);
	}
}

function DrawHeader( canvas Canvas )
{

	local GameReplicationInfo GRI;
	local float XL, YL;
	local font CanvasFont;

	Canvas.DrawColor = WhiteColor;
	GRI = PlayerPawn(Owner).GameReplicationInfo;

	Canvas.Font = MyFonts.GetHugeFont(Canvas.ClipX);

	Canvas.bCenter = True;
	Canvas.StrLen("Test", XL, YL);
	ScoreStart = 0;
	//ScoreStart = 58.0/768.0 * Canvas.ClipY;
	CanvasFont = Canvas.Font;

	if ( GRI.GameEndedComments != "" )
	{

		Canvas.DrawColor = GoldColor;
		Canvas.SetPos(0, ScoreStart);
		DrawShadowText(Canvas,GRI.GameEndedComments, true, true);

	}

	else
	{

		Canvas.SetPos(0, ScoreStart);
		DrawVictoryConditions(Canvas);

	}

	Canvas.bCenter = False;
	Canvas.Font = CanvasFont;

}

function DrawTrailer( canvas Canvas )
{
	local int Hours, Minutes, Seconds;
	local float XL, YL;
	local font CanvasFont;

	if ( Canvas == None ) return;
	CanvasFont = Canvas.Font;
	Canvas.bCenter = true;
	Canvas.StrLen("Test", XL, YL);
	Canvas.DrawColor = WhiteColor;
	Canvas.Font = MyFonts.GetSmallFont(800);

	if(PlayerOwner == None)
		PlayerOwner = PlayerPawn(Owner);

	Canvas.SetPos(0, Canvas.ClipY - 2 * YL);

		if(Level.Author != "")
			DrawShadowText(Canvas, Level.Title @ Author @ Level.Author, true, true);
		else
			DrawShadowText(Canvas, Level.Title, true, true);

	Canvas.SetPos(0, Canvas.ClipY -  YL);

	if ( bTimeDown || (PlayerOwner.GameReplicationInfo.RemainingTime > 0) )
	{
		bTimeDown = true;
		if ( PlayerOwner.GameReplicationInfo.RemainingTime <= 0 )
			DrawShadowText(Canvas, RemainingTime@"00:00"@ " [clock: " $ Level.Hour $ ":" $ TwoDigitString(Level.Minute) $ "]", true, true);
		else
		{
			Minutes = PlayerOwner.GameReplicationInfo.RemainingTime/60;
			Seconds = PlayerOwner.GameReplicationInfo.RemainingTime % 60;
			DrawShadowText(Canvas, RemainingTime@TwoDigitString(Minutes)$":"$TwoDigitString(Seconds)@ " [clock: " $ Level.Hour $ ":" $ TwoDigitString(Level.Minute) $ "]", true, true);
		}
	}

	else
	{
		Seconds = PlayerOwner.GameReplicationInfo.ElapsedTime;
		Minutes = Seconds / 60;
		Hours   = Minutes / 60;
		Seconds = Seconds - (Minutes * 60);
		Minutes = Minutes - (Hours * 60);
		DrawShadowText(Canvas, ElapsedTime@TwoDigitString(Hours)$":"$TwoDigitString(Minutes)$":"$TwoDigitString(Seconds)@ " [clock: " $ Level.Hour $ ":" $ TwoDigitString(Level.Minute) $ "]", true, true);
	}

	if ( PlayerOwner.GameReplicationInfo.GameEndedComments != "" )
	{
		Canvas.bCenter = true;
		Canvas.StrLen("Test", XL, YL);
		Canvas.SetPos(0, Canvas.ClipY - Min(YL*6, Canvas.ClipY * 0.1));
		Canvas.DrawColor = GreenColor;
		DrawShadowText(Canvas, Ended, true, true);
	}

	else if ( (PlayerOwner != None) && (PlayerOwner.Health <= 0) )
	{
		Canvas.bCenter = true;
		Canvas.StrLen("Test", XL, YL);
		Canvas.SetPos(0, Canvas.ClipY - Min(YL*6, Canvas.ClipY * 0.1));
		Canvas.DrawColor = GreenColor;
		DrawShadowText(Canvas, Restart, true, true);
	}

	Canvas.bCenter = false;

}

function ShowScores( canvas Canvas )
{
	local PlayerReplicationInfo PRI;
	local UTES_CInfo UTES_PRI;
	local int PlayerCount, i, ii;
	local float XL, YL, Scale;
	local float YOffset, YStart;
	local font CanvasFont;
	local int ident;

	if (PlayerOwner == None)
	{
		if (PlayerPawn(Owner) != None)
			PlayerOwner = PlayerPawn(Owner);
	}

	Canvas.Style = ERenderStyle.STY_Normal;
	DrawSpectators(Canvas);

	Canvas.SetPos(0, 0);
	DrawHeader(Canvas);

	for ( i=0; i<32; i++ )
		Ordered[i] = None;

	for ( i=0; i<32; i++ )
	{

		PRI = PlayerOwner.GameReplicationInfo.PRIArray[i];
		if(PRI != None)
		{
			if(!PRI.bIsSpectator || PRI.bWaitingPlayer)
			{
				Ordered[PlayerCount] = PRI;
				PlayerCount++;
				UTES_PRI = FindPlayerInfo( PRI.PlayerID);
			}
		}

	}

	SortScores(PlayerCount);
	CanvasFont = Canvas.Font;
	Canvas.Font = MyFonts.GetBigFont(Canvas.ClipX);
	Canvas.StrLen( "TEST", XL, YL );
	YOffset = YL * 3;

	if ( PlayerCount > 15 )
		PlayerCount = FMin(PlayerCount, (Canvas.ClipY - YStart)/YL - 1);
	Canvas.SetPos(0, 0);

	for ( I=0; I<PlayerCount; I++ )
	{

		if ( YOffset < Canvas.ClipY - 34 )
		{
			DrawNameAndPing( Canvas, Ordered[i], 0, YOffset, false );
			YOffset += 34;
		}
		else
		{
			ii++;
		}
		
	}

	Canvas.DrawColor = WhiteColor;

	if ( ii > 0 )
	{
		Canvas.bCenter = True;
		Canvas.Font = MyFonts.GetSmallFont(Canvas.ClipX);
		Canvas.StrLen( ii, XL, YL );
		Canvas.SetPos(0, Canvas.ClipY - YL);
		if ( ii == 1 )
			Canvas.DrawText("(1 player not showing.)", false );
		else
			Canvas.DrawText("("$ii$" players not showing.)", false );
		Canvas.bCenter = False;
	}
	DrawTrailer(Canvas);
}

function DrawNameAndPing(Canvas Canvas, PlayerReplicationInfo PRI, float XOffset, float YOffset, bool bCompressed)
{
	local float XL, YL, XL2, YL2, XL3, YL3;
	local Font CanvasFont;
	local int i, Time, Eff;
	local UTES_CInfo RI;
	local String ImportTexture;
	local Texture FLag;
	local Pawn P;

	if ( Canvas == None )
		return;
	if ( Pri == None )
		return;
	else
		RI = FindPlayerInfo( PRI.PlayerID);

	if ( PlayerOwner == None )
	{
		if ( PlayerPawn(Owner) != None )
			PlayerOwner = PlayerPawn(Owner);
	}
	if (PlayerOwner == None)
		return;
	if ( RI == none )
		return;
	if ( OwnerRI == none && RI.Owner == PlayerOwner )
		OwnerRI = RI;
	Canvas.Style = ERenderStyle.STY_Normal;
	Canvas.DrawColor = WhiteColor;

	Canvas.SetPos(Canvas.ClipX * 0.5 - 390, YOffset );
	DrawShadowTile(Canvas, Texture'shade2', 774, 1, 0, 0, 1, 1);

	Canvas.SetPos(Canvas.ClipX * 0.5 - 390, YOffset );
	DrawShadowTile(Canvas, Texture'shade2', 1, 34, 0, 0, 1, 1);

	Canvas.SetPos(Canvas.ClipX * 0.5 - 390, YOffset + 34);
	DrawShadowTile(Canvas, Texture'shade2', 774, 1, 0, 0, 1, 1);

	Canvas.SetPos(Canvas.ClipX * 0.5 + 384, YOffset );
	DrawShadowTile(Canvas, Texture'shade2', 1, 35, 0, 0, 1, 1);

	Canvas.DrawColor.R = 60;
	Canvas.DrawColor.G = 60;
	Canvas.DrawColor.B = 60;
	Canvas.Style = ERenderStyle.STY_Modulated;
	Canvas.SetPos(Canvas.ClipX * 0.5 - 388, YOffset + 2);
	Canvas.DrawTile(Texture'shade2', 772, 32, 0, 0, 1, 1);

	if ((PlayerOwner.PlayerReplicationInfo.bWaitingPlayer) || (PRI.TalkTexture == None))
	{
		Canvas.DrawColor = WhiteColor;
		Canvas.Style = ERenderStyle.STY_Normal;
		Canvas.SetPos( Canvas.ClipX * 0.5 - 389 , YOffset + 1 );
		Canvas.DrawTile(Texture'faceless', 32, 32, 0, 0, 64, 64);
	}

	else if ((!PlayerOwner.PlayerReplicationInfo.bIsSpectator) || (PRI.TalkTexture != None))
	{
		Canvas.DrawColor = WhiteColor;
		Canvas.Style = ERenderStyle.STY_Normal;
		Canvas.SetPos(Canvas.ClipX * 0.5 - 389, YOffset + 2);
		Canvas.DrawIcon(PRI.TalkTexture,0.5 * 64.0 / PRI.TalkTexture.VSize);
	}
/*
	//Draw Client Rank:
	Canvas.DrawColor = WhiteColor;
	Canvas.Style = ERenderStyle.STY_Normal;
	Canvas.SetPos( Canvas.ClipX * 0.5 - 255 , YOffset + 2 );
	Canvas.DrawRect(Texture'Icon_Rank', 32, 32);
*/
	Canvas.DrawColor = WhiteColor;
	Canvas.Style = ERenderStyle.STY_Normal;

	//Draw Country Flag:
	Canvas.SetPos(Canvas.ClipX * 0.5 + 336, YOffset + 2);
	if (PRI != None)
	{
		if ( !PRI.bIsABot )
		{
			if (RI.Country != "" )
			{
				if ( RI.CachedFlag == None )
				{
					ImportTexture = "MH_A_V2.Flags."$RI.Country;
					RI.CachedFlag = texture(DynamicLoadObject(ImportTexture, class'Texture'));
				}
				if ( RI.CachedFlag == None )
					RI.Country = "";
				else
					Flag = RI.CachedFlag;
			}
		}
		else
		{
			Canvas.Style = ERenderStyle.STY_Translucent;
			Flag = Texture'BotFlag';
		}
	}
	if ( Flag == none )
		Flag = Texture'NoFlag';
	Canvas.DrawTile( Flag, 48, 32, 0, 0, 48, 32);

	Canvas.Font = MyFonts.GetSmallestFont(800);
	//Canvas.SetPos(Canvas.ClipX * 0.5 + 154, YOffset + 2);
	Canvas.SetPos(Canvas.ClipX * 0.5 + 304, YOffset + 2);
	Canvas.Style = ERenderStyle.STY_Normal;
	Canvas.DrawColor = WhiteColor;

	if ( PRI != None )
	{
		if (!PRI.bIsABot)
		{
			if( Level.NetMode != NM_Standalone )
			{
				if (PRI.Ping > 300)
					Canvas.DrawTile(Texture'Icon_Con1', 32, 32, 0, 0, 128, 128);
				else if (PRI.Ping > 150 )
					Canvas.DrawTile(Texture'Icon_Con2', 32, 32, 0, 0, 128, 128);
				else
					Canvas.DrawTile(Texture'Icon_Con3', 32, 32, 0, 0, 128, 128);
				Canvas.Style = ERenderStyle.STY_Normal;
				Canvas.DrawColor = WhiteColor;
				Canvas.StrLen(PRI.Ping, XL, YL );
				Canvas.SetPos(Canvas.ClipX * 0.5 + 321 - 0.5 * XL, YOffset + YL + 4);
				DrawShadowText(Canvas, PRI.Ping, false, true);
			}
			Canvas.Font = MyFonts.GetSmallFont(800);
			Time = Max(0, (Level.TimeSeconds + OwnerRI.timeDelta + OwnerRI.JoinTime - RI.JoinTime)/(Level.TimeDilation*60));//realtime
			Canvas.DrawColor = ParchmentColor;
			Canvas.StrLen("T:    ", XL, YL);
			Canvas.SetPos( Canvas.ClipX * 0.5 + 165, YOffset + YL + 1);
			DrawShadowText(Canvas,":"@Time, false, true);

			Canvas.DrawColor = BronzeColor;
			Canvas.SetPos( Canvas.ClipX * 0.5 + 165, YOffset + 2);
			DrawShadowText(Canvas,"Time", false, true);
	}
	else
	{
		if ( PRI.Ping <= 0 )
		{
			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.DrawTile(Texture'Icon_cpu', 32, 32, 0, 0, 64, 64);
		}
		else
		{
			if( Level.NetMode != NM_Standalone )
			{
				if (PRI.Ping > 300)
					Canvas.DrawTile(Texture'Icon_Con1', 32, 32, 0, 0, 128, 128);
				else if (PRI.Ping > 150 )
					Canvas.DrawTile(Texture'Icon_Con2', 32, 32, 0, 0, 128, 128);
				else
					Canvas.DrawTile(Texture'Icon_Con3', 32, 32, 0, 0, 128, 128);
				Canvas.Style = ERenderStyle.STY_Normal;
				Canvas.DrawColor = WhiteColor;
				Canvas.StrLen(PRI.Ping, XL, YL );
				Canvas.SetPos(Canvas.ClipX * 0.5 + 321 - 0.5 * XL, YOffset + YL + 4);
				DrawShadowText(Canvas, PRI.Ping, false, true);
			}
		}
//		Time for Bot ? Why not ?
		Canvas.Font = MyFonts.GetSmallFont(800);
		if ( Pawn(Owner) != None && Pawn(Owner).PlayerReplicationInfo != None && Pri != None )
			Time = Max(0, (Level.TimeSeconds + Pawn(Owner).PlayerReplicationInfo.StartTime - PRI.StartTime)/(Level.TimeDilation*60));
//		Time = Max(0, (Level.TimeSeconds + OwnerRI.timeDelta + OwnerRI.JoinTime - RI.JoinTime)/(Level.TimeDilation*60));//realtime
		Canvas.DrawColor = ParchmentColor;
		Canvas.StrLen("T:    ", XL, YL);
		Canvas.SetPos( Canvas.ClipX * 0.5 + 165, YOffset + YL + 1);
		DrawShadowText(Canvas,":"@Time, false, true);

		Canvas.DrawColor = BronzeColor;
		Canvas.SetPos( Canvas.ClipX * 0.5 + 165, YOffset + 2);
		DrawShadowText(Canvas,"Time", false, true);
	}
	}

	if (PRI.bAdmin)
		Canvas.DrawColor = WhiteColor;
	else if (PRI.PlayerName == PlayerOwner.PlayerReplicationInfo.PlayerName)
		Canvas.DrawColor = GoldColor;
	else
		Canvas.DrawColor = CyanColor;

	Canvas.Style = ERenderStyle.STY_Normal;
	Canvas.Font = MyFonts.GetSmallFont(800);
	Canvas.SetPos(Canvas.ClipX * 0.5 - 350, YOffset + 2 );
	DrawShadowText(Canvas, PRI.PlayerName, false, true);
	Canvas.StrLen( "0000", XL, YL );

	Canvas.DrawColor = LightGreenColor;
	Canvas.SetPos( Canvas.ClipX * 0.5 - 30, YOffset +2 );
	DrawShadowText(Canvas,"Headshots: "$RI.Headshots, false, true);

	Canvas.DrawColor = RedColor;
	Canvas.SetPos( Canvas.ClipX * 0.5 + 70, YOffset + YL + 1);
	DrawShadowText(Canvas,"Suicides: "$RI.Suicides, false, true);

	Canvas.DrawColor = RedColor;
	Canvas.SetPos( Canvas.ClipX * 0.5 - 30, YOffset + YL + 1);
	DrawShadowText(Canvas,"Lives: "$int(PRI.Deaths), false, true);

	Canvas.DrawColor = LightGreenColor;
	Canvas.SetPos( Canvas.ClipX * 0.5 + 70, YOffset + 2);
	DrawShadowText(Canvas,"Telefrags: "$RI.Telefrags, false, true);

	Canvas.Font = MyFonts.GetMediumFont(600);
	Canvas.DrawColor = ParchmentColor;
	//Canvas.SetPos(Canvas.ClipX * 0.5 -130, YOffset + 2);
	Canvas.SetPos(Canvas.ClipX * 0.5 -160, YOffset + YL - 7);
	DrawShadowText(Canvas,"Score:"@int(PRI.Score), false, true);


	Canvas.Font = MyFonts.GetSmallFont(800);
//	if (!PRI.bIsABot)
//	{
		if ( PRI.Score < 0 )
			Eff = 0;
		else
			Eff = PRI.Score / (PRI.Score + PRI.Deaths) * 100;

		Canvas.SetPos( Canvas.ClipX * 0.5 + 225, YOffset + YL + 1 );
		if (Eff < 20 )
			Canvas.DrawColor = RedColor;
		else if (Eff < 66 )
			Canvas.DrawColor = GoldColor;
		else
			Canvas.DrawColor = GreenColor;
		DrawShadowText(Canvas, "EFF:"@Eff$"%", false, true);

		Canvas.DrawColor = SilverColor;
		Canvas.SetPos( Canvas.ClipX * 0.5 + 225, YOffset + 2 );
		DrawShadowText(Canvas, FPHString$":"@int(60 * PRI.Score/Time), false, true);
//	}

	Canvas.DrawColor = LightCyanColor;
	Canvas.Font = MyFonts.GetSmallestFont(800);//Canvas.ClipX

	//Draw Country Text:
	Canvas.SetPos( Canvas.ClipX * 0.5 - 350, YOffset + YL + 2);
	if (!PRI.bIsABot)
	{
		if ( RI != None && RI.NewVar != "" )
			DrawShadowText(Canvas,"Country: "$RI.NewVar, false, true);
		else
			DrawShadowText(Canvas,"Getting Country", false, true);
	}
	else
		DrawShadowText(Canvas,"Local Pawn", false, true);
}

function SortScores(int N)
{
	local int I, J, Max;
	local PlayerReplicationInfo TempPRI;

	for ( I=0; I<N-1; I++ )
	{
		Max = I;
		for ( J=I+1; J<N; J++ )
		{
			if ( Ordered[J].Score > Ordered[Max].Score )
				Max = J;
			else if ((Ordered[J].Score == Ordered[Max].Score) && (Ordered[J].Deaths < Ordered[Max].Deaths))
				Max = J;
			else if ((Ordered[J].Score == Ordered[Max].Score) && (Ordered[J].Deaths == Ordered[Max].Deaths) &&
					 (Ordered[J].PlayerID < Ordered[Max].Score))
				Max = J;
		}

		TempPRI = Ordered[Max];
		Ordered[Max] = Ordered[I];
		Ordered[I] = TempPRI;
	}
}

function DrawSpectators(Canvas Canvas)
{

	local float XL, YL;
	local Color OldColor;
	local PlayerReplicationInfo PRI;
	local int i;

	Canvas.Font = Canvas.SmallFont;
	Canvas.StrLen("SPECTATORS:", XL, YL);
	Canvas.SetPos(Canvas.ClipX-XL-2, Canvas.ClipY/15);
	DrawShadowText(Canvas, "SPECTATORS:", False, true);


	foreach AllActors(class'PlayerReplicationInfo', PRI)
		if(PRI.bIsSpectator && !PRI.bWaitingPlayer && PRI.PlayerName != "Player")
			Spectators[i++]=PRI.PlayerName;
	while(i<32)
		Spectators[i++]="";


	if(Spectators[0] == "")
	{

		Canvas.StrLen("NONE", XL, YL);
		Canvas.SetPos(Canvas.ClipX-XL-2, Canvas.CurY);
		DrawShadowText(Canvas, "NONE", False, true);

	}
	else
	{

		Canvas.DrawColor=GreenColor;

		for(i=0;i<32;i++)
		{

			if(Spectators[i] == "")
				break;

			Canvas.StrLen(Spectators[i], XL, YL);
			Canvas.SetPos(Canvas.ClipX-XL-2, Canvas.CurY);
			DrawShadowText(Canvas, Spectators[i], False, true);

		}

	}

}

function DrawVictoryConditions(Canvas Canvas)
{

	local TournamentGameReplicationInfo TGRI;
	local float XL, YL;

	TGRI = TournamentGameReplicationInfo(PlayerPawn(Owner).GameReplicationInfo);

	if ( TGRI == None )
		return;

	DrawShadowText(Canvas, TGRI.GameName, true);
	Canvas.StrLen("Test", XL, YL);
	Canvas.SetPos(0, Canvas.CurY - YL);

	Canvas.Font = MyFonts.GetMediumFont( Canvas.ClipX );
	Canvas.StrLen("Test", XL, YL);

	if( TGRI.FragLimit > 0 && TGRI.TimeLimit > 0)
	{

		DrawShadowText(Canvas, FragGoal@TGRI.FragLimit$" / "$TimeLimit@TGRI.TimeLimit$":00", true);
		Canvas.SetPos(0, Canvas.CurY - YL);

	}

	else if ( TGRI.FragLimit > 0 )
	{

		DrawShadowText(Canvas, FragGoal@TGRI.FragLimit, true);
		Canvas.SetPos(0, Canvas.CurY - YL);

	}

	else if ( TGRI.TimeLimit > 0 )
	{

		DrawShadowText(Canvas,TimeLimit@TGRI.TimeLimit$":00", true);
		Canvas.SetPos(0, Canvas.CurY - YL);

	}
}

function DrawShadowText (Canvas Canvas, coerce string Text, optional bool Param,optional bool bSmall, optional bool bGrayShadow)
{

	local Color OldColor;
	local float XL,YL;
	local float X, Y;

	OldColor = Canvas.DrawColor;

	if (bGrayShadow)
	{

		Canvas.DrawColor.R = 127;
		Canvas.DrawColor.G = 127;
		Canvas.DrawColor.B = 127;

	}

	else
	{

		Canvas.DrawColor.R = 0;
		Canvas.DrawColor.G = 0;
		Canvas.DrawColor.B = 0;

	}

	if (bSmall)
	{

		XL = 1;
		YL = 1;

	}

	else
	{

		XL = 2;
		YL = 2;

	}

	X=Canvas.CurX;
	Y=Canvas.CurY;
	Canvas.SetPos(X+XL,Y+YL);
	Canvas.DrawText(Text, Param);
	Canvas.DrawColor = OldColor;
	Canvas.SetPos(X,Y);
	Canvas.DrawText(Text, Param);
}

function DrawShadowTile( Canvas Canvas, Texture Tex, float XL, float YL, float U, float V, float UL, float VL)
{
	local float X, Y;
	local color OldColor;

	X = Canvas.CurX;
	Y = Canvas.CurY;
	Canvas.CurX += 1;
	Canvas.CurY += 1;
	OldColor = Canvas.DrawColor;
	Canvas.DrawColor = BlackColor;
	Canvas.DrawTile(Tex, XL, YL, U, V, UL, VL);
	Canvas.SetPos(X, Y);
	Canvas.DrawColor = OldColor;
	Canvas.DrawTile(Tex, XL, YL, U, V, UL, VL);
}

function UTES_CInfo FindPlayerInfo( int PlayerID)
{
	local UTES_CInfo Result;

	Result = Hashed[ PlayerID % 32];
	while ( Result != none )
	{
		if ( Result.PlayerID == PlayerID )
			return Result;
		result = Result.HashNext;
	}
	//return None; //Unreal c++ defaults return as ZERO, equivalent to 'none' in unreascript
}

/*
function UTES_CInfo FindInfo (PlayerReplicationInfo PRI, out int ident)
{

	local int i;
	local UTES_CInfo RI;
	local bool bFound;

   		for (i=0;i<Index;i++)
   		{

       			if (PlayerInfo[i].PRI == PRI)
       			{

       				ident = i;
           				return PlayerInfo[i].RI;

       			}

   		}

   	foreach AllActors(class'UTES_CInfo',RI)
   	{

      		if (RI.PlayerID == PRI.PlayerID)
      		{

        			bFound = true;
        			break;

      		}

    	}

   	if (!bFound)
       		return None;

   	if (Index < 32)
   	{

       		InitInfo(Index, PRI, RI);
      			ident = Index;
       		Index++;
       		return RI;

   	}

   	else
   	{

       		for (i=0;i<32;i++)
       		{

           			if (PlayerInfo[i].RI == None)
               		break;

       		}

       		InitInfo(i, PRI, RI);
       		ident=i;
       		return RI;
   	}
   	ident = -1;
   	return None;
}
*/
function InitInfo (int i, PlayerReplicationInfo PRI, UTES_CInfo RI)
{
	PlayerInfo[i].PRI = PRI;
	PlayerInfo[i].RI = RI;
	if (PRI == PlayerOwner.PlayerReplicationInfo)
		OwnerRI = RI;

}

static final function string FormatScore(int Time)
{
	if(int(Time % 60) < 10)//fill up a leading 0 to single-digit seconds
		return Time/60 $ ":0" $ int(Time%60);
	else
		return Time/60 $ ":" $ int(Time%60);
}

defaultproperties
{
   GreenColor=(G=255)
   WhiteColor=(R=255,G=255,B=255)
   GoldColor=(R=255,G=255)
   BlueColor=(B=255)
   LightCyanColor=(R=128,G=255,B=255)
   SilverColor=(R=138,G=164,B=166)
   BronzeColor=(R=203,G=147,B=52)
   CyanColor=(G=128,B=255)
   RedColor=(R=255)
   LightGreenColor=(G=136)
   DarkGreenColor=(G=255,B=128)
   ParchmentColor=(R=227,G=210,B=176)
   FrigginDarkColor=(R=4,G=4,B=4)
   PurpleColor=(R=102,G=45,B=145)
}
